from email.policy import default
import click
import boto3
from botocore.config import Config as Boto3Config
from click.decorators import command
from click.termui import prompt
import os
from croudtech_ecs_tools.ecs import Ecs
import json



class EcsTools:
    _services = {}
    _tasks = {}

    def __init__(self, region):
        self.region = region
        print(self.region)
        self.ecs_client = boto3.client("ecs", config=Boto3Config(
            region_name= self.region
        ))

    @property
    def clusters(self):
        if not hasattr(self, "_clusters"):
            paginator = self.ecs_client.get_paginator("list_clusters")
            response_iterator = paginator.paginate(
                PaginationConfig={
                    "PageSize": 10,
                }
            )
            self._clusters = []
            for page in response_iterator:
                for cluster in page["clusterArns"]:
                    self._clusters.append(cluster.split("/").pop())
            
        return self._clusters
    
    def extractFromArn(self, arn):
        arn_parts = arn.split(":")[-1].split("/")[1:]
        return arn_parts

    def get_services(self, cluster):
        if cluster not in self._services:
            self._services[cluster] = []
            paginator = self.ecs_client.get_paginator("list_services")

            response_iterator = paginator.paginate(
                cluster=cluster,           
                PaginationConfig={
                    "PageSize": 50,
                }
            )
            for page in response_iterator:
                for service in page["serviceArns"]:
                    self._services[cluster].append(service)
        return self._services[cluster]

    def get_tasks(self, cluster, service):
        task_key = cluster+service
        if task_key not in self._tasks:
            self._tasks[task_key] = []
            paginator = self.ecs_client.get_paginator("list_tasks")
            response_iterator = paginator.paginate(
                cluster=cluster,
                serviceName=service,           
                PaginationConfig={
                    "PageSize": 50,
                }
            )
            for page in response_iterator:
                for task in page["taskArns"]:
                    self._tasks[task_key].append(task)
        return self._tasks[task_key]

    def describe_task(self, cluster, task_arn):
        response = self.ecs_client.describe_tasks(
            cluster=cluster,
            tasks=[
                task_arn,
            ],            
        )
        task = response["tasks"].pop()
        return task

    def execute_command(self,cluster, container, task_arn, command="bash"):
        return self.ecs_client.execute_command(
            cluster=cluster,
            container=container["name"],
            command=command,
            interactive=True,
            task=task_arn
        )

    def restart_service(self, service_arn, wait=False):
        try:
            cluster, service = self.extractFromArn(service_arn)
        except ValueError as err:
            click.echo(f"Invalid service ARN {service_arn}")
            return
        waiter = self.ecs_client.get_waiter('services_stable')
        
        self.ecs_client.update_service(
            cluster=cluster,
            service=service,
            forceNewDeployment=True
        )
        if wait:
            waiter.wait(
                cluster=cluster,
                services=[
                    service,
                ],            
                WaiterConfig={
                    'Delay': 5,
                    'MaxAttempts': 100
                }
            )
        return True

    def get_task_containers(self, cluster, task_arn):
        return self.describe_task(cluster, task_arn)["containers"]
    
    def get_service_options(self, cluster):
        options = []
        for index, option in enumerate(self.get_services(cluster)):
            option_name = option.split("/").pop()
            options.append(f"{index}: {option_name}")
        return "\n".join(options)

    def get_task_options(self, cluster, service):
        options = []
        for index, option in enumerate(self.get_tasks(cluster, service)):
            option_name = option.split("/").pop()
            options.append(f"{index}: {option_name}")
        return "\n".join(options)

    def get_task__container_options(self, cluster, task_arn):
        options = []
        for index, option in enumerate(self.get_task_containers(cluster, task_arn)):
            option_name = option["name"]
            options.append(f"{index}: {option_name}")
        return "\n".join(options)

    def get_cluster_options(self):
        options = []
        for index, option in enumerate(self.clusters):
            options.append(f"{index}: {option}")
        return "\n".join(options)

@click.group()
@click.version_option()
def cli():
    "Tools for managing ECS Services and Tasks"


@cli.command()
@click.option("--region", required=True, default=os.getenv("AWS_DEFAULT_REGION", "eu-west-2"))
@click.option("--command", default="bash")
def ecs_shell(region, command):
    ecs_tools = EcsTools(region)

    "Shell into an ECS task container"
    click.secho(ecs_tools.get_cluster_options(), fg="cyan")
    cluster = ecs_tools.clusters[int(click.prompt("Please select a cluster"))]

    click.secho(ecs_tools.get_service_options(cluster), fg="cyan")
    service_arn = ecs_tools.get_services(cluster)[int(click.prompt("Please select a service"))]

    click.secho(ecs_tools.get_task_options(cluster, service_arn), fg="cyan")
    task_arn = ecs_tools.get_tasks(cluster, service_arn)[int(click.prompt("Please select a task"))]

    click.echo(ecs_tools.get_task__container_options(cluster, task_arn))
    container = ecs_tools.get_task_containers(cluster, task_arn)[int(click.prompt("Please select a container"))]["name"]
    click.secho("Connecting to  Cluster:" + cluster + " Service:" + service_arn.split("/").pop() + " Task:" + task_arn.split("/").pop() + " Container: " + container, fg="green" )
    task_id = task_arn.split("/").pop()
    command = f"aws ecs execute-command --cluster {cluster} --task {task_id} --container {container} --interactive --command {command}"
    click.secho("Executing command", fg="green")
    click.secho(command, fg="cyan")
    os.system(command)

@cli.command()
@click.option("--region", required=True, default=os.getenv("AWS_DEFAULT_REGION", "eu-west-2"))
@click.option('--wait/--no-wait', default=False, help="Wait for service to become stable before exiting")
@click.argument("service_arn", required=False)
def restart_service(region, wait, service_arn):
    ecs_tools = EcsTools(region)
    if not service_arn: 
        
        click.secho(ecs_tools.get_cluster_options(), fg="cyan")
        cluster = ecs_tools.clusters[int(click.prompt("Please select a cluster"))]

        click.secho(ecs_tools.get_service_options(cluster), fg="cyan")
        service_arn = ecs_tools.get_services(cluster)[int(click.prompt("Please select a service"))]
    
    click.echo(f"Restarting ARN: {service_arn}")
    ecs_tools.restart_service(service_arn, wait)
    if wait:
        click.echo(f"Service {service_arn} restarted")


@cli.command()
@click.option("--cluster", required=True)
def list_service_discovery_endpoints(cluster):
    ecs_manager = Ecs(cluster=cluster)
    print(json.dumps(ecs_manager.list_ecs_service_endpoints(), indent=2, default=str))

if __name__ == "__main__":
    cli()


import boto3
from mypy_boto3_ecs.type_defs import ServiceTypeDef
from mypy_boto3_servicediscovery.type_defs import NamespaceTypeDef
from typing import List
import json

def parse_arn(arn):
    elements = arn.split(':')
    result = {'arn': elements[0],
            'partition': elements[1],
            'service': elements[2],
            'region': elements[3],
            'account': elements[4]
           }
    if len(elements) == 7:
        result['resourcetype'], result['resource'] = elements[5:]
    elif '/' not in elements[5]:
        result['resource'] = elements[5]
        result['resourcetype'] = None
    else:
        result['resourcetype'], result['resource'] = elements[5].split('/')
    return result

def chunk_list(list_to_chunk, size):
    return [list_to_chunk[i * size:(i + 1) * size] for i in range((len(list_to_chunk) + size - 1) // size )]

class Ecs:
    def __init__(self, cluster):
        self.cluster_name = cluster        

    @property
    def ecs_client(self):
        if not hasattr(self, "_ecs_client"):
            self._ecs_client = boto3.client("ecs")
        return self._ecs_client

    @property
    def servicediscovery_client(self):
        if not hasattr(self, "_servicediscovery_client"):
            self._servicediscovery_client = boto3.client("servicediscovery")
        return self._servicediscovery_client

    @property
    def services(self) -> List[ServiceTypeDef]:
        if not hasattr(self, "_services"):
            self._services = []
            for service_arns in chunk_list(self.service_arns, 10):
                self._services = self._services + self.ecs_client.describe_services(cluster=self.cluster_name, services=service_arns)["services"]  
        return self._services

    @property
    def namespaces(self) -> List[NamespaceTypeDef]:
        if not hasattr(self, "_namespaces"):
            self._namespaces = {}
            paginator = self.servicediscovery_client.get_paginator("list_namespaces")
            response_iterator = paginator.paginate()
            for page in response_iterator:
                for namespace in page["Namespaces"]:
                    self._namespaces[namespace["Id"]] = namespace
        return self._namespaces

    @property
    def service_arns(self):
        if not hasattr(self, "_service_arns"):
            self._service_arns = []
            paginator = self.ecs_client.get_paginator("list_services")
            response_iterator = paginator.paginate(
                cluster=self.cluster_name,                
            )
            for page in response_iterator:
                self._service_arns = self._service_arns + page["serviceArns"]
        return self._service_arns

    def list_ecs_service_endpoints(self):
        self._ecs_service_endpoints = {}        
        for service in self.services:
            for service_registry_arn in service["serviceRegistries"]:
                sd = self.servicediscovery_client.get_service(Id=parse_arn(service_registry_arn["registryArn"])["resource"])
                hostname = ".".join([sd["Service"]["Name"], self.namespaces[sd["Service"]["NamespaceId"]]["Name"]])
                self._ecs_service_endpoints[hostname] = sd["Service"]["Name"]

        return self._ecs_service_endpoints




from setuptools import setup
import os

VERSION = os.getenv("SEMVER", os.getenv("GitVersion_FullSemVer", "dev"))


def get_long_description():
    with open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"),
        encoding="utf8",
    ) as fp:
        return fp.read()


setup(
    name="croudtech-ecs-tools",
    description="Tools for managing ECS Services and Tasks",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Jim Robinson",
    url="https://github.com/CroudTech/croudtech-ecs-tools",
    project_urls={
        "Issues": "https://github.com/CroudTech/croudtech-ecs-tools/issues",
        "CI": "https://github.com/CroudTech/croudtech-ecs-tools/actions",
        "Changelog": "https://github.com/CroudTech/croudtech-ecs-tools/releases",
    },
    license="Apache License, Version 2.0",
    version=VERSION,
    packages=["croudtech_ecs_tools"],
    entry_points="""
        [console_scripts]
        croudtech-ecs-tools=croudtech_ecs_tools.cli:cli
    """,
    install_requires=[
        "boto3==1.20.28",
        "botocore==1.23.28; python_version >= '3.6'",
        "click==8.0.3",
        "jmespath==0.10.0; python_version >= '2.6' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "python-dateutil==2.8.2; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "s3transfer==0.5.0; python_version >= '3.6'",
        "six==1.16.0; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "urllib3==1.26.7; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4' and python_version < '4'",
    ],
    extras_require={
        "test": ["pytest"]
    },
    python_requires=">=3.8",
)
